#!/bin/bash
# Wrapper for the vmsn start script so that environment can be properly set

###################### OS detection ##########################################

if [ -f "/etc/os-release" ]
then
    FLAVOR=$(grep "^ID=" "/etc/os-release" | cut -d'=' -f2)
else
    FLAVOR=$OSTYPE
fi

NASBOX=0
if test -e /bin/get_key_value && \
   test -e /etc/synoinfo.conf && \
   /bin/get_key_value /etc/synoinfo.conf unique | grep -q "synology"
then
    NASBOX=1
    FLAVOR=synology
fi

if test -e /etc/model.conf && \
   head /etc/model.conf | grep -q "VENDOR = QNAP"
then
    NASBOX=1
    FLAVOR=qnap
fi

if test -e /proc/readynas/model && \
   head  /proc/readynas/model | grep -q "ReadyNAS"
then
    NASBOX=1
    FLAVOR=readynas
fi

if_os () { [[ $OSTYPE == *$1* ]]; }
if_flavor () { [[ $FLAVOR == *$1* ]]; }

###################### Test for root user ####################################

GROUP=`id -u`
if test $GROUP -ne 0; then
    echo "Archiware Pure can only be started by a root user!" >&2
    exit 1
fi
USER=`whoami`

###################### Export the environment ################################

dir=`dirname "$0"`
cwd=`pwd`;cd "$dir"
VMSNDIR=`pwd`
PATH="$VMSNDIR/bin":/usr/local/bin:$PATH
LD_LIBRARY_PATH="$VMSNDIR/bin":"$VMSNDIR/vix/lib64":$LD_LIBRARY_PATH
export VMSNDIR PATH LD_LIBRARY_PATH NASBOX FLAVOR PORT TELNET USER
export -f if_os if_flavor

###################### Load kernel module ####################################

if [ "$NASBOX" = 1 ] && [ "$FLAVOR" = "synology" ]
then
    NBD=$(lsmod | grep nbd)
    if test -z "$NBD"; then
        case "`uname -r`" in
            3.10.77*)
                case "`cat /etc/synoinfo.conf | grep unique`" in
                    *broadwell*)
                        insmod $VMSNDIR/bin/synbd_3_10_77_broadwell.ko
                    ;;
                    *avoton*)
                        insmod $VMSNDIR/bin/synbd_3_10_77_avoton.ko
                    ;;
                    *grantley*)
                        insmod $VMSNDIR/bin/synbd_3_10_77_grantley.ko
                    ;;					
                    *bromolow*)
                        insmod $VMSNDIR/bin/synbd_3_10_77_bromolow.ko
                    ;;					
                    *braswell*)
                        insmod $VMSNDIR/bin/synbd_3_10_77_braswell.ko
                    ;;					
                    *cedarview*)
                        insmod $VMSNDIR/bin/synbd_3_10_77_cedarview.ko
                    ;;
                    *x64*)
                        insmod $VMSNDIR/bin/synbd_3_10_77_x64.ko
                    ;;
                    *)
                        echo "Unsupported Synology kernel detected. 'Restore file' feature will not work." > $SYNOPKG_TEMP_LOGFILE
                    ;;
                esac					
            ;;
            3.10.102*)
                case "`cat /etc/synoinfo.conf | grep unique`" in
                    *broadwell*)
                        insmod $VMSNDIR/bin/synbd_3_10_102_broadwell.ko
                    ;;
                    *avoton*)
                        insmod $VMSNDIR/bin/synbd_3_10_102_avoton.ko
                    ;;
                    *grantley*)
                        insmod $VMSNDIR/bin/synbd_3_10_102_grantley.ko
                    ;;					
                    *bromolow*)
                        insmod $VMSNDIR/bin/synbd_3_10_102_bromolow.ko
                    ;;					
                    *braswell*)
                        insmod $VMSNDIR/bin/synbd_3_10_102_braswell.ko
                    ;;					
                    *cedarview*)
                        insmod $VMSNDIR/bin/synbd_3_10_102_cedarview.ko
                    ;;
                    *x64*)
                        insmod $VMSNDIR/bin/synbd_3_10_102_x64.ko
                    ;;
                    *)
                        echo "Unsupported Synology kernel detected. 'Restore file' feature will not work." > $SYNOPKG_TEMP_LOGFILE
                    ;;
                esac
            ;;
            3.10.105*)
                case "`cat /etc/synoinfo.conf | grep unique`" in
                    *broadwell*)
                        insmod $VMSNDIR/bin/synbd_3_10_105_broadwell.ko
                    ;;
                    *avoton*)
                        insmod $VMSNDIR/bin/synbd_3_10_105_avoton.ko
                    ;;
                    *grantley*)
                        insmod $VMSNDIR/bin/synbd_3_10_105_grantley.ko
                    ;;					
                    *bromolow*)
                        insmod $VMSNDIR/bin/synbd_3_10_105_bromolow.ko
                    ;;					
                    *braswell*)
                        insmod $VMSNDIR/bin/synbd_3_10_105_braswell.ko
                    ;;					
                    *cedarview*)
                        insmod $VMSNDIR/bin/synbd_3_10_105_cedarview.ko
                    ;;
                    *x64*)
                        insmod $VMSNDIR/bin/synbd_3_10_105_x64.ko
                    ;;
                    *)
                        echo "Unsupported Synology kernel detected. 'Restore file' feature will not work." > $SYNOPKG_TEMP_LOGFILE
                    ;;
                esac
            ;;
            4.4.15*)
                case "`cat /etc/synoinfo.conf | grep unique`" in
                    *denverton*)
                        insmod $VMSNDIR/bin/synbd_4_4_15_denverton.ko
                    ;;
                    *apollolake*)
                        insmod $VMSNDIR/bin/synbd_4_4_15_apollolake.ko
                    ;;
                    *)
                        echo "Unsupported Synology kernel detected. 'Restore file' feature will not work." > $SYNOPKG_TEMP_LOGFILE
                    ;;
                esac
            ;;
            4.4.59*)
                case "`cat /etc/synoinfo.conf | grep unique`" in
                    *denverton*)
                        insmod $VMSNDIR/bin/synbd_4_4_59_denverton.ko
                    ;;
                    *apollolake*)
                        insmod $VMSNDIR/bin/synbd_4_4_59_apollolake.ko
                    ;;
                    *broadwellnk*)
                        insmod $VMSNDIR/bin/synbd_4_4_59_broadwellnk.ko
                    ;;					
                    *)
                        echo "Unsupported Synology kernel detected. 'Restore file' feature will not work." > $SYNOPKG_TEMP_LOGFILE
                    ;;
                esac
            ;;
            *)
                echo "Unsupported Synology kernel detected. 'Restore file' feature will not work." > $SYNOPKG_TEMP_LOGFILE
            ;;
        esac
    fi
fi

if [ "$NASBOX" = 1 ] && [ "$FLAVOR" = "readynas" ]
then
    NBD=$(lsmod | grep nbd)
    if test -z "$NBD"; then
        case "`uname -r`" in
            4.4.98*)
                insmod $VMSNDIR/bin/rnnbd_4_4_98.ko
            ;;
            4.4.111*)
                insmod $VMSNDIR/bin/rnnbd_4_4_111.ko
            ;;
            4.4.116*)
                insmod $VMSNDIR/bin/rnnbd_4_4_116.ko
            ;;
            4.4.140*)
                insmod $VMSNDIR/bin/rnnbd_4_4_140.ko
            ;;
            4.4.157*)
                insmod $VMSNDIR/bin/rnnbd_4_4_157.ko
            ;;
            4.4.163*)
                insmod $VMSNDIR/bin/rnnbd_4_4_163.ko
            ;;
            4.4.178*)
                insmod $VMSNDIR/bin/rnnbd_4_4_178.ko
            ;;
            *)
            ;;
        esac
    fi
fi
###################### Configure port if args given ##########################

case $# in
    0)
    if [ "$NASBOX" = 1 ]
    then
        if_flavor synology && PORT=20500
        if_flavor qnap && PORT=8060
        if_flavor readynas && PORT=8250
        TELNET=$(($PORT + 1000))
        sed -i "/ns_param address 0.0.0.0/{n;s/.*/    ns_param port $PORT/}" $VMSNDIR/etc/vmsn.tcl
        sed -i "/ns_param address 127.0.0.1/{n;s/.*/        ns_param port $TELNET/}" $VMSNDIR/etc/vmsn.tcl
    fi
    ;;
    1)
    if [ -n "$(printf '%s\n' $1 | sed 's/[0-9]//g')" ]; then
        echo '$1 is not numeric'
        exit 1
    fi
    PORT=$1
    TELNET=$(($PORT + 1000))
    sed -i "/ns_param address 0.0.0.0/{n;s/.*/    ns_param port $PORT/}" $VMSNDIR/etc/vmsn.tcl
    sed -i "/ns_param address 127.0.0.1/{n;s/.*/        ns_param port $TELNET/}" $VMSNDIR/etc/vmsn.tcl
    ;;
    *)
    echo "Usage: $0 [port]"
    exit 1
    ;;
esac

######################## Start Archiware Pure ################################

exec ./start_server.sh