#!/bin/bash
#
# VM Storage Node startup script.
#
# See the file "license.txt" for information on usage and
# redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
# ----------------------------------------------------------------------
#



###################### Configure autostart ###################################

if [ "$NASBOX" = 0 ]
then
    if_os linux && SERVICEFILE=/lib/systemd/system/archiware-pure.service
    if_flavor suse && SERVICEFILE=/usr/lib/systemd/system/archiware-pure.service

    if test ! -r $SERVICEFILE; then
        echo "Arranging for server to start/stop at power cycle"
        echo "[Unit]" > $SERVICEFILE
        echo "Description=Archiware Pure" >> $SERVICEFILE
        echo "After=multi-user.target" >> $SERVICEFILE
        echo "[Service]" >> $SERVICEFILE
        echo "ExecStart=$VMSNDIR/start.sh" >> $SERVICEFILE
        echo "TimeoutStartSec=infinity" >> $SERVICEFILE
        echo "ExecStop=$VMSNDIR/stop.sh" >> $SERVICEFILE
        echo "TimeoutStopSec=300s" >> $SERVICEFILE
        echo "Type=forking" >> $SERVICEFILE
        echo "KillMode=mixed" >> $SERVICEFILE
        echo "Restart=no" >> $SERVICEFILE
        echo "User=root" >> $SERVICEFILE
        echo "Group=root" >> $SERVICEFILE
        echo "PIDFile=$VMSNDIR/log/vmsn.pid" >> $SERVICEFILE    
        echo "[Install]" >> $SERVICEFILE
        echo "WantedBy=multi-user.target" >> $SERVICEFILE
        systemctl daemon-reload
        systemctl enable archiware-pure
    fi
fi

###################### Configure PAM #########################################


PAMFILE=/etc/pam.d/nsd
PAMPATH=pam_unix.so
if_flavor qnap && PAMPATH=/usr/local/lib/pam_unix.so

if test ! -r $PAMFILE; then
    echo "# VMSN installation: `/bin/date`" > $PAMFILE
    echo "auth  required        $PAMPATH" >> $PAMFILE
    echo "account       required        $PAMPATH" >> $PAMFILE
fi

###################### Silently auto update ##################################


UPDATEFILE=$(ls -t "$VMSNDIR/temp" | grep -E '^vmsn\.[0-9]{1,2}\.[0-9]{1,2}\.[0-9]{1,3}\.tgz'| head -1)
if [ -f "$VMSNDIR/temp/$UPDATEFILE" ] ; then
    echo "Updating Archiware Pure..."
    tar xzf "$VMSNDIR/temp/$UPDATEFILE" -C "$VMSNDIR" > /dev/null 2>&1
    rm -f "$VMSNDIR/temp/$UPDATEFILE"
    echo "Update finished."
fi

###################### Start the app server ##################################

echo "Starting Archiware Pure application server, be patient..."

pid=`cat "$VMSNDIR/log/vmsn.pid" 2>/dev/null`

if test $? -eq 0; then
    kill -s 0 $pid 2>/dev/null
    if test $? -eq 0; then
        echo " Archiware Pure (already running) (pid=$pid)"
        exit 0
    fi
    rm -f "$VMSNDIR/log/vmsn.pid"
fi

"$VMSNDIR/bin/nsd" -w -u $USER -t etc/vmsn.tcl 1>/tmp/vmsn.$$ 2>&1
if test $? -ne 0; then
    echo " Archiware Pure failed to start; see /tmp/vmsn.$$ for more info."
    exit 1
fi

rm -f /tmp/vmsn.$$

ntry=12

while test $ntry -gt 0; do
    grep -E "nsmain: NaviServer/[0-9]+.[0-9]+.[0-9dabrc]+ running" $VMSNDIR/log/vmsn.log 1>/dev/null 2>&1
    case $? in
        0)
            sleep 1
            pid=`cat "$VMSNDIR/log/vmsn.pid" 2>/dev/null`
            echo "  Archiware Pure pid: $pid - running."
            ntry=1
            break
            ;;
        *)
            ;;
    esac
    sleep 5
    ntry=`expr $ntry - 1`
    if test $ntry -eq 0; then
        echo " Archiware Pure pid: $pid - failed to start; see $VMSNDIR/log/vmsn.log for more info."
        exit 1
    fi
done


echo "Started Archiware Pure application server."
exit 0

