#!/bin/bash
#
# VM Storage Node stop script.
#
# See the file "license.txt" for information on usage and
# redistribution of this file, and for a DISCLAIMER OF ALL WARRANTIES.
#
# ----------------------------------------------------------------------
#

###################### OS detection ##########################################

if [ -f "/etc/os-release" ]
then
    FLAVOR=$(grep "^ID=" "/etc/os-release" | cut -d'=' -f2)
else
    FLAVOR=$OSTYPE
fi

NASBOX=0
if test -e /bin/get_key_value && \
   test -e /etc/synoinfo.conf && \
   /bin/get_key_value /etc/synoinfo.conf unique | grep -q "synology"
then
    NASBOX=1
    FLAVOR=synology
fi

if test -e /etc/model.conf && \
   head /etc/model.conf | grep -q "VENDOR = QNAP"
then
    NASBOX=1
    FLAVOR=qnap
fi

if test -e /proc/readynas/model && \
   head  /proc/readynas/model | grep -q "ReadyNAS"
then
    NASBOX=1
    FLAVOR=readynas
fi

if_os () { [[ $OSTYPE == *$1* ]]; }
if_flavor () { [[ $FLAVOR == *$1* ]]; }

###################### Test for root user ####################################

if test `id -u` -ne 0; then
    echo "Archiware Pure can only be stopped by a root user!" >&2
    exit 1
fi

###################### Stop the app server ###################################


if_os linux && ps="ps -eo pid,args"

#
# Get the home directory
#

dir=`dirname "$0"`
cwd=`pwd`;cd "$dir"
VMSNDIR=`pwd`;export VMSNDIR
cd "$cwd"

echo "Stopping Archiware Pure application server, be patient..."
pid=`cat "$VMSNDIR"/log/vmsn.pid 2>/dev/null`
if test $? -ne 0; then
    echo "  Archiware Pure pid: $pid (not running)"
else
    ntry=30
    while test $ntry -gt 0; do
        kill -s SIGTERM $pid 2>/dev/null
        if test $? -eq 0; then
            echo "  Archiware Pure pid: $pid (stopped)"
            rm -f "$VMSNDIR"/log/vmsn.pid
            break
        fi
        sleep 2
        ntry=`expr $ntry - 1`
    done
    if test $ntry -eq 0; then
        echo "  Archiware Pure pid: $pid (failed)"
        rm -f "$VMSNDIR"/log/vmsn.pid
    fi
fi
echo "Stopped Archiware Pure application server."

#
# Handle any left-overs
#

sleep 2
pids=""
proc="$VMSNDIR/bin/nsd"

for pid in `$ps | grep "$proc" | grep -v grep | awk '{print $1;}'`; do
    kill -9 $pid 2>/dev/null
    pids="$pids $pid"
done
if test ! -z "$pids"; then
    pids=""
    sleep 2
    for pid in `$ps | grep "$proc" | grep -v grep | awk '{print $1;}'`; do
        pids="$pids $pid"
    done
    if test ! -z "$pids"; then
        echo "Could not stop \"$proc\" pid(s): $pids" >&2
        exit 133
    fi
fi

exit 0

